"""
LDA, Load Accumulator Test

This is one of the Memory Operations in the c6502
"""

import unittest

from nesasm.compiler import lexical, syntax, semantic


class LdaTest(unittest.TestCase):
    def test_lda_imm(self):
        tokens = list(lexical('LDA #$10'))
        self.assertEqual(2, len(tokens))
        self.assertEqual('T_INSTRUCTION', tokens[0]['type'])
        self.assertEqual('T_HEX_NUMBER', tokens[1]['type'])
        ast = syntax(tokens)
        self.assertEqual(1, len(ast))
        self.assertEqual('S_IMMEDIATE', ast[0]['type'])
        code = semantic(ast)
        self.assertEqual(code, [0xA9, 0x10])

    def test_lda_imm_with_decimal(self):
        tokens = list(lexical('LDA #10'))
        self.assertEqual(2, len(tokens))
        self.assertEqual('T_INSTRUCTION', tokens[0]['type'])
        self.assertEqual('T_DECIMAL_NUMBER', tokens[1]['type'])
        ast = syntax(tokens)
        self.assertEqual(1, len(ast))
        self.assertEqual('S_IMMEDIATE', ast[0]['type'])
        code = semantic(ast)
        self.assertEqual(code, [0xA9, 0x0A])

    def test_lda_imm_with_binary(self):
        tokens = list(lexical('LDA #%00000100'))
        self.assertEqual(2, len(tokens))
        self.assertEqual('T_INSTRUCTION', tokens[0]['type'])
        self.assertEqual('T_BINARY_NUMBER', tokens[1]['type'])
        ast = syntax(tokens)
        self.assertEqual(1, len(ast))
        self.assertEqual('S_IMMEDIATE', ast[0]['type'])
        code = semantic(ast)
        self.assertEqual(code, [0xA9, 0x04])

    def test_lda_zp(self):
        tokens = list(lexical('LDA $00'))
        self.assertEqual(2, len(tokens))
        self.assertEqual('T_INSTRUCTION', tokens[0]['type'])
        self.assertEqual('T_ADDRESS', tokens[1]['type'])
        ast = syntax(tokens)
        self.assertEqual(1, len(ast))
        self.assertEqual('S_ZEROPAGE', ast[0]['type'])
        code = semantic(ast)
        self.assertEqual(code, [0xA5, 0x00])

    def test_lda_zpx(self):
        tokens = list(lexical('LDA $10,X'))
        self.assertEqual(4, len(tokens))
        self.assertEqual('T_INSTRUCTION', tokens[0]['type'])
        self.assertEqual('T_ADDRESS', tokens[1]['type'])
        self.assertEqual('T_SEPARATOR', tokens[2]['type'])
        self.assertEqual('T_REGISTER', tokens[3]['type'])
        ast = syntax(tokens)
        self.assertEqual(1, len(ast))
        self.assertEqual('S_ZEROPAGE_X', ast[0]['type'])
        code = semantic(ast)
        self.assertEqual(code, [0xB5, 0x10])

    def test_lda_abs(self):
        tokens = list(lexical('LDA $1234'))
        self.assertEqual(2, len(tokens))
        self.assertEqual('T_INSTRUCTION', tokens[0]['type'])
        self.assertEqual('T_ADDRESS', tokens[1]['type'])
        ast = syntax(tokens)
        self.assertEqual(1, len(ast))
        self.assertEqual('S_ABSOLUTE', ast[0]['type'])
        code = semantic(ast)
        self.assertEqual(code, [0xAD, 0x34, 0x12])

    def test_lda_absx(self):
        tokens = list(lexical('LDA $1234,X'))
        self.assertEqual(4, len(tokens))
        self.assertEqual('T_INSTRUCTION', tokens[0]['type'])
        self.assertEqual('T_ADDRESS', tokens[1]['type'])
        self.assertEqual('T_SEPARATOR', tokens[2]['type'])
        self.assertEqual('T_REGISTER', tokens[3]['type'])
        ast = syntax(tokens)
        self.assertEqual(1, len(ast))
        self.assertEqual('S_ABSOLUTE_X', ast[0]['type'])
        code = semantic(ast)
        self.assertEqual(code, [0xBD, 0x34, 0x12])

    def test_lda_absy(self):
        tokens = list(lexical('LDA $1234,Y'))
        self.assertEqual(4, len(tokens))
        self.assertEqual('T_INSTRUCTION', tokens[0]['type'])
        self.assertEqual('T_ADDRESS', tokens[1]['type'])
        self.assertEqual('T_SEPARATOR', tokens[2]['type'])
        self.assertEqual('T_REGISTER', tokens[3]['type'])
        ast = syntax(tokens)
        self.assertEqual(1, len(ast))
        self.assertEqual('S_ABSOLUTE_Y', ast[0]['type'])
        code = semantic(ast)
        self.assertEqual(code, [0xB9, 0x34, 0x12])

    def test_lda_indx(self):
        tokens = list(lexical('LDA ($20,X)'))
        self.assertEqual(6, len(tokens))
        self.assertEqual('T_INSTRUCTION', tokens[0]['type'])
        self.assertEqual('T_OPEN', tokens[1]['type'])
        self.assertEqual('T_ADDRESS', tokens[2]['type'])
        self.assertEqual('$20', tokens[2]['value'])
        self.assertEqual('T_SEPARATOR', tokens[3]['type'])
        self.assertEqual('T_REGISTER', tokens[4]['type'])
        self.assertEqual('T_CLOSE', tokens[5]['type'])
        ast = syntax(tokens)
        self.assertEqual(1, len(ast))
        self.assertEqual('S_INDIRECT_X', ast[0]['type'])
        code = semantic(ast)
        self.assertEqual(code, [0xA1, 0x20])

    def test_lda_indy(self):
        tokens = list(lexical('LDA ($20),Y'))
        self.assertEqual(6, len(tokens))
        self.assertEqual('T_INSTRUCTION', tokens[0]['type'])
        self.assertEqual('T_OPEN', tokens[1]['type'])
        self.assertEqual('T_ADDRESS', tokens[2]['type'])
        self.assertEqual('T_CLOSE', tokens[3]['type'])
        self.assertEqual('T_SEPARATOR', tokens[4]['type'])
        self.assertEqual('T_REGISTER', tokens[5]['type'])
        ast = syntax(tokens)
        self.assertEqual(1, len(ast))
        self.assertEqual('S_INDIRECT_Y', ast[0]['type'])
        code = semantic(ast)
        self.assertEqual(code, [0xB1, 0x20])
